$NOLIST
$NOSB
$NOMOD51
$INCLUDE (REG152.ASM)
$LIST

;*********************************************************************
;THIS PROGRAM IS WRITTEN TO RECIEVE A TRANSMISSION IN CSMA/CD MODE 
;UNDER CPU CONTROL AND ASSUMES SOME RECEPTIONS WILL BE IN ERROR, THUS 
;THE NAME Recieve Error Under Cpu Control (REUCC2).
;
;ASSUMPTIONS:
;   THE DESTINATION OF RECEIVED DATA IS IN EXTERNAL RAM AND THE STARTING 
;   ADDRESS IS DEFINED BY:
;               MSG_SOURCE (CAN BE UPTO 16 BITS)
;
;   THE STARTING LOCATION WHERE THE DATA IS STORED WILL ALWAYS BEGIN AT 
;   THE SAME ADDRESS FOR EACH NEW PACKET (THE OLD DATA IS WRITTEN OVER)
;       THE NUMBER OF BYTES RECEIVED IS STORED AT 400H AND 401H
;       THE MESSAGE BEGINS AT 402H
;
;   8-BIT ADDRESSING
;
;   16-BIT CRC
;
;   IF A RECEPTION ERROR OCCURS THE ACTIONS TAKEN ARE:
;       LOG THE ERROR 
;       RE-INITIALIZE POINTERS AND COUNTERS
;       DISCARD OLD DATA (WRITTEN OVER)
;       ENABLE RECEIVER.
;
;INTERNAL RAM LOCATIONS USED AND MODIFIED:
;
;           0FAH TO 0FFH    NUMBER OF ALIGNMENT ERRORS
;           0F4H TO 0F9H    NUMBER OF CRC ERRORS
;           0EEH TO 0F3H    NUMBER OF OVERRUN ERRORS
;           0E8H TO 0EDH    NUMBER OF TRANSMISSION ABORT ERRORS
;           0E2H TO 0E7H    NUMBER OF UNKNOWN RECEPTION ERRORS
;
;           7FH             MSG_COUNTER_HIGH
;           7EH             MSG_COUNTER_LOW
;           7DH             MSG_POINTER_HIGH
;           7CH             MSG_POINTER_LOW
;
;SFRS USED AND MODIFIED:
;           ACC
;           ADR0
;           ADR1
;           ADR2
;           ADR3
;           BAUD
;           DPL
;           DPH
;           GMOD
;           IE          
;           IEN1
;           RFIFO
;           RSTAT
;
;REGISTERS USED AND MODIFIED:
;           R0
;           R7
;
;BITS USED AND MODIFIED:
;           C
;
;*********************************************************************

$EJ
;*********************************************************************
;THIS PROGRAM CAN BE BROKEN INTO 4 SEPARATE SECTIONS.  THESE ARE: THE
;INTERRUPT VECTORS (LOCATIONS 0 TO 0FFH), GSC INTERRUPT SERVICE
;SOFTWARE (LOCATIONS 100H TO 1FFH), GSC INITIALIZATION SOFTWARE 
;(LOCATIONS 200H TO 2FFH), AND THOSE SECTIONS OF CODE NEEDED TO MAKE 
;THE PROGRAM FUNCTION (LOCATIONS 300H TO 3FFH).  THIS SHOULD ALLOW A 
;USER TO PULL OUT ONLY THE SECTIONS THAT ARE NEEDED TO COMPLETE A 
;SPECIFIC APPLICATION.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN TO GO ALONG WITH THE INTEL FAE TRAINING
;MATERIAL FOR THE 8XC152 INTERMEDIATE TRAINING (JAN 1987) BUT HOPEFULLY
;IS SIMPLE ENOUGH FOR USERS WHO HAVE NOT TAKEN THE TRAINING COURSE TO 
;UNDERSTAND. THE LETTERS IN PARENTHESES AND FOLLOWED BY A HYPHEN, (A) -, 
;REFER TO THE LETTERS IN FLOW CHARTS GIVEN DURING THE FAE TRAINING
;*********************************************************************

$EJ

ERROR_POINTER       EQU     R0

MSG_COUNTER_HIGH    DATA    7FH
MSG_COUNTER_LOW     DATA    (MSG_COUNTER_HIGH - 1)

MSG_POINTER_HIGH    DATA    (MSG_COUNTER_LOW - 1)
MSG_POINTER_LOW     DATA    (MSG_POINTER_HIGH - 1)

AE_COUNT            DATA    0FFH
CRCE_COUNT          DATA    (AE_COUNT-6)
OVR_COUNT           DATA    (CRCE_COUNT-6)
RCABT_COUNT         DATA    (OVR_COUNT-6)
UNKNOWN_COUNT       DATA    (RCABT_COUNT-6)
NEXT_LOCATION       DATA    (UNKNOWN_COUNT-6)

MSG_DEST            EQU 402H        ;STARTING LOCATION OF RECEIVED
                                    ;DATA

ADDRESS0            EQU     11H
ADDRESS1            EQU     ADDRESS0
ADDRESS2            EQU     ADDRESS0
ADDRESS3            EQU     ADDRESS0

$EJ

ORG 0
    JMP INIT1

ORG 2BH
REC_VALID:
    JMP VALID_REC

ORG 33H
REC_ERROR:
    JMP ERROR_REC

$EJ

ORG 100H
VALID_REC:
    MOV A,RFIFO                     ;GET RECEIVED DATA

    MOV DPL,MSG_POINTER_LOW
    MOV DPH,MSG_POINTER_HIGH        ;MOVE POINTER ADDRESS INTO DPTR

    MOVX @DPTR,A                    ;STORE DATA

    INC DPTR
    MOV MSG_POINTER_LOW,DPL
    MOV MSG_POINTER_HIGH,DPH        ;INCREMENT MESSAGE POINTER

    MOV A,MSG_COUNTER_LOW
    ADD A,#1
    MOV MSG_COUNTER_LOW,A           ;INCREMENT LOW BYTE OF MSG COUNTER

    MOV A,MSG_COUNTER_HIGH
    ADDC A,#0
    MOV MSG_COUNTER_HIGH,A          ;INCREMENT HIGH BYTE IF LOW BYTE 
                                    ;INCREMENT RESULT IN CARRY

;*********************************************************************
; THIS SECTION OF CODE WAS INITIALLY PUT IN TO MAKE SURE THAT THE 
; POINTER THAT INDICATES WHERE THE INCOMING DATA WAS TO BE STORED DID 
; NOT OVERFLOW FROM 0FFFFH TO 00H AND WRITE OVER DATA THAT SHOULD BE
; SAVED. THIS WAS CONSIDERED A RESTRICTION THAT NOT ALL USERS MAY WANT
; SO THIS SECTION OF CODE WAS COMMENTED OUT. IF A USER WISHES TO USE
; THIS FUNCTION THE SEMI-COLONS NEED TO BE REMOVED AND THE CODE FOLLOWING
; THE LABEL "POINTER_OVERFLOW" WILL NEED TO BE MODIFIED TO IMPLEMENT
; WHATEVER ACTIONS THE USERS WISHES TO TAKE IN THE EVENT THE POINTERS
; DO OVERFLOW.
;*********************************************************************
;   CLR A
;   CJNE A,MSG_POINTER_LOW,END_RECEIVE_VALID
;   CJNE A,MSG_POINTER_HIGH,END_RECEIVE_VALID
;                                   ;CHECK THAT POINTER HAS NOT OVERFLOWED
;                                   ;AND POINTS TO ADDRESS 0
;
;   NOP
;
;POINTER_OVERFLOW:
;   JMP POINTER_OVERFLOW            ;LOOP IF POINTER = 0
;*********************************************************************

END_RECEIVE_VALID:
    JB RFNE,VALID_REC               ;STORE NEXT BYTE IF RFIFO NOT EMPTY
    RETI

ERROR_REC:
;*********************************************************************
; (D) - DISPOSE OF DATA (IN THIS CASE DATA IS WRITTEN OVER)
; (E) - UPDATE POINTERS AND COUNTERS
;*********************************************************************

    MOV MSG_POINTER_LOW,#LOW (MSG_DEST)
    MOV MSG_POINTER_HIGH,#HIGH (MSG_DEST)
                                    ;REINITIALIZE POINTER

    MOV MSG_COUNTER_LOW,#0
    MOV MSG_COUNTER_HIGH,#0         ;REINITIALIZE COUNTER

    JNB LNI,$                       ;MAKE SURE TRANSMISSIONS HAVE
                                    ;STOPPED

    CALL INC_ERROR_COUNT            ;LOG ERROR TYPE

;*********************************************************************
; (F) - ENABLE RECEIVER
;*********************************************************************
    SETB GREN                       ;ENABLE RECIEVER, CLEAR RFIFO

    RETI

$EJ

ORG 200H
INIT2:
;*********************************************************************
; USER GENERIC GSC RECEIVER INITIALIZATION
;*********************************************************************

    MOV BAUD,#60                    ;INIT BAUD RATE FOR # DIVISABLE BY 6

    MOV GMOD,#02                    ;INIT FOR CSMA/CD, 8-BIT PREAMBLE (N/A
                                    ;FOR RECEIVER), 16 CRC, 8 BIT ADDR

    MOV ADR0,#ADDRESS0              ;ADDRESSES THAT THIS STATION WILL 
    MOV ADR1,#ADDRESS1              ;RECEIVE (MASK REGS ALREADY = 0
    MOV ADR2,#ADDRESS2              ;FROM RESET)
    MOV ADR3,#ADDRESS3

    MOV MSG_POINTER_LOW,#LOW (MSG_DEST)
    MOV MSG_POINTER_HIGH,#HIGH (MSG_DEST)
                                    ;INITIALIZE WHERE RECEIVED DATA
                                    ;WILL BE STORED

    MOV MSG_COUNTER_LOW,#0
    MOV MSG_COUNTER_HIGH,#0         ;INIT BYTE COUNTER TO 0

    SETB GREN                       ;RECIEVER SHOULD ALWAYS BE ENABLED
                                    ;UNLESS DATA ALREADY RECEIVED WOULD
                                    ;BE WRITTEN OVER BEFORE DEALT WITH
    RET

$EJ

ORG 300H
INIT1:
;*********************************************************************
; USER GENERIC INITIALIZATION - USER SHOULD PLACE INITIALIZING SOFTWARE
; ROUTINES HERE
;*********************************************************************

    SETB EGSRV                      ;ENABLE RECEIVE VALID INTERRUPT
    SETB EGSRE                      ;ENABLE RECEIVE ERROR INTERRUPT
                                    ;THOUGH SHOULD NOT BE USED
    SETB EA

    MOV R0,#NEXT_LOCATION

COUNTER_CLEAR:
    INC R0
    MOV @R0,#0                      ;CLEAR OUT ERROR COUNTER AREA
    CJNE R0,#0FFH,COUNTER_CLEAR     ;LOOP UNTIL ALL COUNTERS = 0
    
    CALL INIT2                      ;GSC INITIALIZATION
    JMP MAIN

INC_ERROR_COUNT:
;*********************************************************************
; THIS ROUTINE INCREMENTS THE ERROR COUNT (UPTO 6 BYTES) FOR EACH TYPE
; OF ERROR DETECTED BY HARDWARE
;
; BECAUSE OTHER ERROR BITS MAY BE SET WHEN OVR IS SET, OVR MUST BE TESTED
; BEFORE AE OR CRCE.  ALSO, IN MOST APPLICATIONS AN ABORT MAY ALSO CAUSE
; AN ALIGNMENT ERROR OR CRC ERROR, AND AN ALIGNMENT ERROR MAY CAUSE A CRC
; ERROR, THE FOLLOWING SEQUENCE OF CHECKING ERROR BITS SHOULD BE FOLLOWED
; TO GET AN  ACCURATE TALLY OF THE TYPES OF ERRORS THAT ARE OCCURRING
;
; COMBINATION OF ERROR BITS I HAVE SEEN:
;   CRCE SET FOR BAD CRC
;   RCABT AND AE SET FOR RCABT (ALIGNMENT ERROR MAY ALSO EXIST)
;   AE SET FOR ALIGNMENT ERROR (CRC WAS BAD ALSO)
;   OVR, AE, CRCE AND RFNE SET FOR OVR (THOUGH CRC IS GOOD AND NO AE)
;
;*********************************************************************

RCABT_CHECK:
    JNB RCABT,OVR_CHECK             ;SEE IF ERROR CAUSED BY RCABT
    MOV ERROR_POINTER,#RCABT_COUNT
    JMP INC_COUNT

OVR_CHECK:
;*********************************************************************
; (A) - CHECK IF OVERRUN (IN FAE TRAINING THE ACTION THAT SHOULD BE 
;       TAKEN WAS TO STOP OTHER TASKS, BUT SINCE THIS PROGRAM ONLY
;       RECEIVES DATA, THERE ARE NO OTHER TASKS TO STOP SO STEP B IS
;       OMITTED)
;*********************************************************************
    JNB OVR,CRC_CHECK               ;SEE IF ERROR CAUSE BY OVR
    MOV ERROR_POINTER,#OVR_COUNT
    JMP INC_COUNT

CRC_CHECK:
    JNB CRCE,AE_CHECK               ;SEE IF ERROR CAUSED BY CRCE
    MOV ERROR_POINTER,#CRCE_COUNT
    JMP INC_COUNT

AE_CHECK:
    JNB AE,UNKNOWN_ERROR            ;SEE IF ERROR CAUSED BY AE
    MOV ERROR_POINTER,#AE_COUNT
    JMP INC_COUNT

UNKNOWN_ERROR:
    MOV ERROR_POINTER,#UNKNOWN_COUNT
                                    ;COUNT OF ERRORS NOT CAUSED BY AE,
                                    ;CRCE, OVR, OR RCABT

INC_COUNT:
;*********************************************************************
; (C) - LOG ERROR
;*********************************************************************

    SETB C                          ;ADD 1 ON FIRST LOOP
    MOV R7,#6                       ;# OF BYTES IN EACH COUNTER FIELD

INC_COUNT_LOOP:
    MOV A,@ERROR_POINTER            ;GET BYTE OF COUNTER
    ADDC A,#0
    MOV @ERROR_POINTER,A
    DEC ERROR_POINTER
    DJNZ R7,INC_COUNT_LOOP
;   JC COUNTER_OVERFLOW             ;OVERFLOW IF CARRY GENERATED. THIS
                                    ;WAS INITIALLY PUT IN TO STOP THE
                                    ;FLOW OF THE PROGRAM IF ANY OF THE
                                    ;ERROR COUNTERS OVERFLOWED WITH THE
                                    ;EXPECTATION THAT THE USER WOULD
                                    ;MODIFY THE CODE TO DUMP THE ERROR
                                    ;COUNT CONTENTS AND RE-INITIALIZE THE
                                    ;COUNTER LOCATIONS.  THIS HAS BEEN
                                    ;COMMENTED OUT BECAUSE SOME USERS
                                    ;MAY NOT WANT TO EVEN TRACK THE NUMBER
                                    ;OF ERRORS AND CONSEQUENTLY WOULD NOT
                                    ;WANT TO MODIFY THE CODE WHEN THE ERROR
                                    ;COUNTERS OVERFLOWED.

    RET

;COUNTER_OVERFLOW:
;   INC ERROR_POINTER               ;POINT TO MSB OF COUNTER FIELD
;   MOV @ERROR_POINTER,#0FFH        ;AND STORE 0FFH
;   INC ERROR_POINTER               ;POINT TO NEXT BYTE OF COUNTER FIELD
;   MOV @ERROR_POINTER,#0FFH        ;AND STORE 0FFH
;   INC ERROR_POINTER               ;POINT TO NEXT BYTE OF COUNTER FIELD
;   MOV @ERROR_POINTER,#0FFH        ;AND STORE 0FFH
;   INC ERROR_POINTER               ;POINT TO NEXT BYTE OF COUNTER FIELD
;   MOV @ERROR_POINTER,#0FFH        ;AND STORE 0FFH
;   INC ERROR_POINTER               ;POINT TO NEXT BYTE OF COUNTER FIELD
;   MOV @ERROR_POINTER,#0FFH        ;AND STORE 0FFH
;   INC ERROR_POINTER               ;POINT TO NEXT BYTE OF COUNTER FIELD
;   MOV @ERROR_POINTER,#0FFH        ;AND STORE 0FFH
;   JMP $                           ;LOOP UNTIL H/W RESETS DEVICE

MAIN:
;*********************************************************************
; USER MAIN PROGRAM LOOP - USER SHOULD PLACE ALL APPLICATION CODE OR
; BRANCHES TO OTHER SECTIONS OF THE APPLICATION CODE HERE.
;*********************************************************************

    JNB GREN,RECEIVER_CHECK         ;IF RECEIVER NOT ENABLED SEE WHY

    JMP MAIN                        ;CONTINOUSLY LOOP

RECEIVER_CHECK:
;*********************************************************************
; THE ONLY WAY GREN CAN BE CLEARED IS IF A VALID MESSAGE HAS COMPLETED
; AND THERE WERE NO ERRORS OR IF AN ERROR OCCURS.  IN CASE OF AN ERROR
; THE ERROR HANDLING S/W SHOULD RE-ENABLE RECEIVER.
;*********************************************************************

;*********************************************************************
; (B) - RDN = 1? (IN THE FAE TRAINING THIS WAS SHOWN TO BE IN THE
;   INTERRUPT SERVICE ROUTINE BUT RDN MAY NOT BE SET BY THE TIME THE
;   LAST BYTE WAS RECEIVED, SO THE CHECK FOR RDN WAS MOVED HERE
;*********************************************************************

    JB RDN,RE_ENABLE_RECEIVER       ;IF DONE RECIEVING, ENABLE RECEIVER

    JNB EGSRE,ERROR_HALT            ;IF RDN=0, THEN ERROR MUST HAVE
                                    ;OCCURRED AND IF EGSRE=0 THEN THINGS
                                    ;ARE REALLY SCREWED UP

    JB OVR,MAIN                     ;IF EGSRE AND OVR = 1, LET INTERRUPT
                                    ;HANDLE IT

    JB RCABT,MAIN                   ;IF EGSRE AND RCABT = 1, LET INTERRUPT
                                    ;HANDLE IT

    JB AE,MAIN                      ;IF EGSRE AND AE = 1, LET INTERRUPT
                                    ;HANDLE IT

    JB CRCE,MAIN                    ;IF EGSRE AND CRCE = 1, LET INTERRUPT
                                    ;HANDLE IT

ERROR_HALT:

    NOP                             ;CYCLE TO SET BREAKPOINT ON ICE
    JMP $                           ;LOOP UNTIL USER DOES SOMETHING
                                    ;ABOUT UNKNOWN CONDITION

RE_ENABLE_RECEIVER:
    JNB EGSRV,ERROR_HALT            ;UNKNOWN CONDITION IF EGSRV DISABLED

    JB RFNE,MAIN                    ;IF RFIFO NOT EMPTY, LET INTERRUPT
                                    ;HANDLE IT

    MOV MSG_POINTER_LOW,#LOW (MSG_DEST)
    MOV MSG_POINTER_HIGH,#HIGH (MSG_DEST)
                                    ;REINITIALIZE POINTER

    MOV DPTR,#(MSG_DEST - 2)
    MOV A,MSG_COUNTER_HIGH
    MOVX @DPTR,A
    INC DPTR
    MOV A,MSG_COUNTER_LOW
    MOVX @DPTR,A                    ;STORE NUMBER OF BYTES RECEIVED

    MOV MSG_COUNTER_LOW,#0
    MOV MSG_COUNTER_HIGH,#0         ;REINITIALIZE COUNTER

    SETB GREN                       ;ENABLE RECEIVER IF RDN=1 AND RFNE=0
    JMP MAIN

END
